from __future__ import absolute_import

import os
import os.path
import sys
import gzip
import shutil
import tarfile
import hashlib
import subprocess
import tempfile
from io import BytesIO
from contextlib import closing

try:
    from urllib import urlopen
except ImportError:
    from urllib.request import urlopen

URL = "https://downloads.xiph.org/releases/%(LIB)s/lib%(LIB)s-%(VERSION)s.tar.gz"

LIBS = {
    # 'lib': [("version", "SHA-256")]
    'ogg': [
        ('1.3.2', "e19ee34711d7af328cb26287f4137e70630e7261b17cbe3cd41011d73a654692"),
    ],
    'vorbis': [
        ('1.3.5', "6efbcecdd3e5dfbf090341b485da9d176eb250d893e3eb378c428a2db38301ce"),
    ]
}


OPTS = {
    'ogg': [],
    'vorbis': [],
}


def run(*cmd):
    print("Executing %s ..." % cmd[0])
    if subprocess.call(list(cmd)):
        raise RuntimeError("Command %s failed" % cmd[0])


def cmmi(basedir, installdir, *config_opts):
    orig_dir = os.getcwd()
    try:
        os.chdir(basedir)
        run(os.path.join(basedir, "configure"),
            "--enable-static", "--disable-shared", "--disable-dependency-tracking",
            "--prefix", installdir,
            *config_opts)
        run("make", "-j9")
        run("make", "install")
    finally:
        os.chdir(orig_dir)


def build(ogg_version=None, vorbis_version=None, installdir=None):
    if not installdir:
        installdir = tempfile.mkdtemp(suffix="-install")

    libs = [
        ('ogg', ogg_version, []),
        ('vorbis', vorbis_version, ['--disable-docs', '--disable-examples']),
    ]

    opts = []
    for lib, version, extra_opts in libs:
        if not version:
            check = LIBS[lib][0]
            version = check[0]
        else:
            for check in LIBS[lib]:
                if check[0] == version:
                    break
            else:
                check = None

        url = URL % {'LIB': lib, 'VERSION': version}
        with closing(urlopen(url)) as dl:
            data = dl.read()
        if check is not None:
            if hashlib.sha256(data).hexdigest() != check[1]:
                raise RuntimeError("checksum mismatch for lib %s" % lib)

        tempdir = tempfile.mkdtemp()
        try:
            zf = tarfile.TarFile(fileobj=gzip.GzipFile(fileobj=BytesIO(data)))
            zf.extractall(tempdir)
            cmmi(os.path.join(tempdir, "lib%s-%s" % (lib, version)),
                 installdir, *(opts + extra_opts))
            if os.path.isdir(os.path.join(installdir, "share")):
                shutil.rmtree(os.path.join(installdir, "share"))
            opts.append("--with-%s=%s" % (lib, installdir))
        finally:
            shutil.rmtree(tempdir)
    return installdir


if __name__ == '__main__':
    env = os.environ
    installdir = build(
        ogg_version=env.get('OGG_VERSION'),
        vorbis_version=env.get('VORBIS_VERSION'),
        installdir=sys.argv[1] if len(sys.argv) > 1 else None,
    )
    print(installdir)



#!/usr/bin/env python

import sys
import os, os.path

if os.path.isdir(os.path.join(os.path.dirname(__file__), 'src')):
    sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'src'))

import pyrogg


def recode(filename, destdir, quality):
    basename   = os.path.basename(filename)
    targetname = os.path.join(destdir, basename)

    print("Encoding '%s' in quality %+d" % (basename, quality))

    recoder = pyrogg.VorbisFileRecoder(filename)
    time = recoder.recode(targetname, quality)
    print("Encoding '%s' in quality %+d took %.2f seconds" % (
          basename, quality, time))


def _process(args):
    filename, dirname, quality = args
    try:
        recode(filename, dirname, quality)
    except (IOError, pyrogg.VorbisException) as e:
        print(e)
        return False
    return True


def main():
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option('-q', '--quality', dest="quality", type=int,
                      help="recode to quality level QUALITY", metavar="QUALITY")
    parser.add_option('-p', '--parallel', dest="parallel", type=int,
                      help="number of files to process in parallel", metavar="THREADS")
    parser.add_option('-d', '--output-dir', dest="output_dir",
                      help="write output files to directory DIR", metavar="DIR")

    parser.set_defaults(quality=2, output_dir=None)

    options, args = parser.parse_args()

    if len(args) == 0:
        print("No input files found")
        return False  # no failure
    if options.output_dir is None:
        print("Output directory is required, call with '-h' for help")
        return True   # incorrect usage

    quality = min(10, max(-1, options.quality))
    dirname = options.output_dir

    print("Recoding %d file%s to target directory '%s' ..." % (
          len(args), 's' if len(args) != 1 else '', dirname))

    if not os.access(dirname, os.F_OK):
        os.makedirs(dirname)

    if not os.access(dirname, os.W_OK):
        print("Cannot write to '%s'" % dirname)

    any_failures = False
    if options.parallel:
        import multiprocessing
        pool = multiprocessing.Pool(options.parallel)
        params = [(filename, dirname, quality) for filename in args]
        try:
            results = pool.map(_process, params)
            for result, filename in zip(results, args):
                if not result:
                    any_failures = True
                    print("Recoding failed for file %s" % filename)
        finally:
            pool.close()
    else:
        for filename in args:
            if not _process([filename, dirname, quality]):
                any_failures = True
                print("Recoding failed for file %s" % filename)
    return any_failures


if __name__ == '__main__':
    sys.exit(1 if main() else 0)



from libc.stdio cimport FILE
from ogg cimport ogg_packet, ogg_int64_t

cdef extern from "vorbis/codec.h" nogil:
    cdef enum VorbisError:
        OV_FALSE      # -1
        OV_EOF        # -2
        OV_HOLE       # -3
        OV_EREAD      # -128
        OV_EFAULT     # -129
        OV_EIMPL      # -130
        OV_EINVAL     # -131
        OV_ENOTVORBIS # -132
        OV_EBADHEADER # -133
        OV_EVERSION   # -134
        OV_ENOTAUDIO  # -135
        OV_EBADPACKET # -136
        OV_EBADLINK   # -137
        OV_ENOSEEK    # -138

    ctypedef struct vorbis_info:
        int version
        int channels
        long rate
        long bitrate_upper
        long bitrate_nominal
        long bitrate_lower
        long bitrate_window

    ctypedef struct vorbis_comment:
        char **user_comments
        int  *comment_lengths
        int  comments
        char *vendor

    ctypedef struct vorbis_dsp_state:
        pass

    ctypedef struct vorbis_block:
        pass

    cdef void vorbis_info_init(vorbis_info* vi)
    cdef void vorbis_info_clear(vorbis_info* vi)

    cdef int vorbis_analysis_init(vorbis_dsp_state *v, vorbis_info *vi)
    cdef int vorbis_analysis_blockout(vorbis_dsp_state *v,vorbis_block *vb)
    cdef int vorbis_analysis(vorbis_block *vb, ogg_packet *op)
    cdef int vorbis_analysis_wrote(vorbis_dsp_state *v, int vals)
    cdef float** vorbis_analysis_buffer(vorbis_dsp_state *v, int vals)
    cdef int vorbis_analysis_headerout(vorbis_dsp_state *v,
                                       vorbis_comment *vc,
                                       ogg_packet *op,
                                       ogg_packet *op_comm,
                                       ogg_packet *op_code)

    cdef int  vorbis_bitrate_addblock(vorbis_block *vb)
    cdef int vorbis_bitrate_flushpacket(vorbis_dsp_state *vd,
                                        ogg_packet *op)

    cdef int  vorbis_block_init(vorbis_dsp_state *v, vorbis_block *vb)
    cdef int  vorbis_block_clear(vorbis_block *vb)
    cdef void vorbis_dsp_clear(vorbis_dsp_state *v)


cdef extern from "vorbis/vorbisfile.h" nogil:

    ctypedef struct OggVorbis_File:
        long current_serialno
        vorbis_info*    vi
        vorbis_comment* vc

    ctypedef struct ov_callbacks:
        size_t (*read_func)  (void *ptr, size_t size, size_t nmemb, void *datasource)
        int    (*seek_func)  (void *datasource, ogg_int64_t offset, int whence)
        int    (*close_func) (void *datasource)
        long   (*tell_func)  (void *datasource)

    cdef int ov_open(FILE *f, OggVorbis_File *vf, char *initial, long ibytes)
    cdef int ov_test(FILE *f, OggVorbis_File *vf, char *initial, long ibytes)
    cdef int ov_test_open(OggVorbis_File *vf)
    cdef int ov_clear(OggVorbis_File *vf)
    cdef long ov_read(OggVorbis_File *vf, char *buffer, int length,
                      int bigendianp, int word, int signed, int *bitstream)

    cdef int ov_open_callbacks(void* datasource, OggVorbis_File *vf, char *initial,
                               long ibytes, ov_callbacks callbacks)


cdef extern from "vorbis/vorbisenc.h" nogil:
    cdef int vorbis_encode_init_vbr(vorbis_info *vi,
                                    long channels, long rate,
                                    float base_quality)



cimport ogg, vorbis

cimport cpython.mem
cimport cpython.exc

cimport cython
from cython cimport parallel

from libc cimport stdio
from libc.errno cimport errno
from libc.string cimport memcpy

cdef object time
from time import time

cdef object sys
import sys

__version__ = "0.3"


class VorbisException(Exception):
    pass

class VorbisDecodeException(VorbisException):
    pass


DEF READ_BUFFER_SIZE = 4 * (256 * 1024)  # 256K samples of 2 bytes stereo

cdef vorbis.ov_callbacks filelike_callbacks
filelike_callbacks.read_func  = _readFilelike
filelike_callbacks.seek_func  = _seekFilelike
filelike_callbacks.tell_func  = _tellFilelike
filelike_callbacks.close_func = _closeFilelike


cdef int _checkVorbisResult(int result_code) except -1:
    if result_code < 0:
        raise VorbisException()
    return 0

cdef bytes _encodeFilename(filename):
    if isinstance(filename, bytes):
        return <bytes>filename
    elif isinstance(filename, unicode):
        encoding = sys.getfilesystemencoding()
        if not encoding:
            encoding = 'iso8859-1'  # just give it a try ...
        return (<unicode>filename).encode(encoding)
    else:
        raise ValueError("expected file path, got %s" % type(filename))


cdef class VorbisFile:
    cdef vorbis.OggVorbis_File _vorbisfile
    cdef object filename

    def __cinit__(self, filename):
        cdef stdio.FILE* cfile
        cdef int result
        filename = _encodeFilename(filename)
        cfile = stdio.fopen(filename, "r")
        if cfile is NULL:
            raise IOError("Opening file %r failed" % filename)
        result = vorbis.ov_test(cfile, &self._vorbisfile, NULL, 0)
        if result == 0:
            result = vorbis.ov_test_open(&self._vorbisfile)
        if result < 0:
            vorbis.ov_clear(&self._vorbisfile) # closes the file
            if result == vorbis.OV_ENOTVORBIS:
                raise VorbisException("%r is not a Vorbis file" % filename)
            raise IOError("Error reading from file %r" % filename)
        self.filename = filename

    def __dealloc__(self):
        if self.filename is not None:
            vorbis.ov_clear(&self._vorbisfile) # closes the file


ctypedef int (*page_writer_function)(void*, ogg.ogg_page*) nogil

cdef struct communication:   # thread communication
    int done
    int buffer_ready
    int next_buffer_ready
    long read_status[2]


cdef class _VorbisRecoder:
    cdef page_writer_function _write

    @cython.final
    cdef _recode(self, void* status, vorbis.OggVorbis_File* vorbisfile,
                 int target_quality):
        cdef int error_status, init_steps
        cdef float cquality
        cdef char* decbuffer = NULL
        cdef ogg.ogg_stream_state stream_state
        cdef vorbis.vorbis_info vorbis_info
        cdef vorbis.vorbis_dsp_state dsp_state
        cdef vorbis.vorbis_block vorbis_block

        if target_quality > 10:
            cquality = 1.0
        elif target_quality < -1:
            cquality = -0.1
        else:
            cquality = target_quality / 10.0

        # encoding setup
        vorbis.vorbis_info_init(&vorbis_info)
        error_status = vorbis.vorbis_encode_init_vbr(
            &vorbis_info, vorbisfile.vi.channels,
            vorbisfile.vi.rate, cquality)
        if not error_status:
            error_status = _initVorbisEncoder(
                &stream_state, &dsp_state, &vorbis_info,
                # use arbitrary but predictable serial number
                vorbisfile.current_serialno | <int>(cquality*10+2))
        if not error_status:
            vorbis.vorbis_block_init(&dsp_state, &vorbis_block)
            error_status = self._writeVorbisHeader(
                status, &stream_state, &dsp_state, vorbisfile.vc)

        t = time()
        try:
            if not error_status:
                decbuffer = <char*> cpython.mem.PyMem_Malloc(READ_BUFFER_SIZE*2)
                if decbuffer is NULL:
                    raise MemoryError()
                self._recodeParallel(
                    status, decbuffer, vorbisfile,
                    dsp_state, vorbis_block, stream_state)
        finally:
            ogg.ogg_stream_clear(&stream_state)
            vorbis.vorbis_block_clear(&vorbis_block)
            vorbis.vorbis_dsp_clear(&dsp_state)
            vorbis.vorbis_info_clear(&vorbis_info)
            cpython.mem.PyMem_Free(decbuffer)
            vorbis.ov_clear(vorbisfile) # closes the input file
        t = time() - t

        if error_status:
            raise VorbisException(
                "Unknown error %d occurred while preparing recoder" %
                error_status)

        return t

    @cython.final
    cdef _recodeParallel(self, void* status, char* decbuffer,
                         vorbis.OggVorbis_File* vorbisfile,
                         vorbis.vorbis_dsp_state dsp_state,
                         vorbis.vorbis_block vorbis_block,
                         ogg.ogg_stream_state stream_state
                         ):
        cdef long read_status, values_read
        cdef int i, eos, buffer_part, current_section
        cdef communication com   # thread communication

        com.read_status[0] = vorbis.ov_read(
            vorbisfile, decbuffer, READ_BUFFER_SIZE,
            0, 2, 1, &current_section)
        _checkVorbisResult(com.read_status[0])

        com.done = 0
        com.buffer_ready = 0

        with nogil, parallel.parallel(num_threads=2):
            while True:
                # using prange() loop to synchronise
                for i in parallel.prange(2):
                    if i == 1:
                        # reader thread
                        buffer_part = 0 if com.buffer_ready == 1 else 1
                        read_status = vorbis.ov_read(
                            vorbisfile,
                            decbuffer + (READ_BUFFER_SIZE * buffer_part),
                            READ_BUFFER_SIZE,
                            0, 2, 1, &current_section)
                        com.read_status[buffer_part] = read_status
                        com.next_buffer_ready = buffer_part
                    else:
                        # writer thread
                        buffer_part = com.buffer_ready
                        read_status = com.read_status[buffer_part]
                        encbuffer = vorbis.vorbis_analysis_buffer(
                            &dsp_state, READ_BUFFER_SIZE/4)
                        if read_status > 0:
                            values_read = read_status / 4
                            _splitStereo(
                                decbuffer + (READ_BUFFER_SIZE * buffer_part),
                                encbuffer[0], encbuffer[1], values_read)
                        else:
                            values_read = 0
                        vorbis.vorbis_analysis_wrote(&dsp_state, values_read)

                        eos = self._encodeVorbisBlocks(
                            status, &stream_state, &dsp_state,
                            &vorbis_block)
                        if eos:
                            com.done = 1
                if com.done:
                    break
                # switch buffers when both are done
                com.buffer_ready = com.next_buffer_ready

        read_status = com.read_status[com.buffer_ready]
        if read_status == vorbis.OV_HOLE or read_status == vorbis.OV_EBADLINK:
            raise VorbisDecodeException("lost sync")

    @cython.final
    cdef int _writeVorbisHeader(self, void* status,
                                 ogg.ogg_stream_state* stream_state,
                                 vorbis.vorbis_dsp_state* dsp_state,
                                 vorbis.vorbis_comment* comment):
        cdef ogg.ogg_packet header
        cdef ogg.ogg_packet header_comm
        cdef ogg.ogg_packet header_code
        cdef ogg.ogg_page   header_page
        cdef int result
        result = vorbis.vorbis_analysis_headerout(
            dsp_state, comment,
            &header, &header_comm, &header_code)
        if result < 0:
            return result
        if ogg.ogg_stream_packetin(stream_state, &header) < 0:
            return -1
        if ogg.ogg_stream_packetin(stream_state, &header_comm) < 0:
            return -1
        if ogg.ogg_stream_packetin(stream_state, &header_code) < 0:
            return -1

        result = ogg.ogg_stream_flush(stream_state, &header_page)
        while result > 0:
            result = self._write(status, &header_page)
            if result < 0:
                return result
            result = ogg.ogg_stream_flush(stream_state, &header_page)
        return 0

    @cython.final
    cdef int _encodeVorbisBlocks(self, void* status,
                                 ogg.ogg_stream_state* stream_state,
                                 vorbis.vorbis_dsp_state* dsp_state,
                                 vorbis.vorbis_block* vorbis_block) nogil:
        cdef int eos, error_status
        eos = 0
        error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
        while error_status == 1:
            error_status = vorbis.vorbis_analysis(vorbis_block, NULL)
            if error_status < 0:
                return error_status
            error_status = vorbis.vorbis_bitrate_addblock(vorbis_block)
            if error_status < 0:
                return error_status
            error_status = self._writeOggPackets(status, stream_state, dsp_state)
            if error_status < 0:
                return error_status
            eos = error_status
            error_status = vorbis.vorbis_analysis_blockout(dsp_state, vorbis_block)
        if error_status < 0:
            return error_status
        return eos

    @cython.final
    cdef int _writeOggPackets(self, void* status,
                              ogg.ogg_stream_state* stream_state,
                              vorbis.vorbis_dsp_state* dsp_state) nogil:
        cdef ogg.ogg_packet ogg_packet
        cdef ogg.ogg_page   ogg_page
        cdef int eos, error_status
        eos = 0
        error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
        while error_status > 0:
            if ogg.ogg_stream_packetin(stream_state, &ogg_packet) < 0:
                return -1
            while ogg.ogg_stream_pageout(stream_state, &ogg_page) != 0 and not eos:
                if self._write(status, &ogg_page) < 0:
                    return -1
                eos = ogg.ogg_page_eos(&ogg_page)
            error_status = vorbis.vorbis_bitrate_flushpacket(dsp_state, &ogg_packet)
        if error_status < 0:
            return error_status
        return eos


cdef class VorbisFileRecoder(_VorbisRecoder):
    cdef object _input_filename

    def __cinit__(self, input_filename):
        self._write = _writeToFile
        self._input_filename = _encodeFilename(input_filename)

    def recode(self, output_filename, int quality):
        cdef vorbis.OggVorbis_File vorbisfile
        cdef stdio.FILE* cinfile
        cdef stdio.FILE* coutfile

        output_filename = _encodeFilename(output_filename)
        cinfile = stdio.fopen(self._input_filename, "r")
        if cinfile is NULL:
            cpython.exc.PyErr_SetFromErrno(IOError)

        result = vorbis.ov_open(cinfile, &vorbisfile, NULL, 0)
        if result == vorbis.OV_ENOTVORBIS:
            stdio.fclose(cinfile)
            raise VorbisException(
                "%r is not a Vorbis file" % self._input_filename)
        if result < 0:
            stdio.fclose(cinfile)
            raise IOError("Error reading from file %r" % self._input_filename)

        coutfile = stdio.fopen(output_filename, "w")
        if coutfile == NULL:
            vorbis.ov_clear(&vorbisfile) # closes the input file
            raise IOError("Error opening output file %r" % output_filename)

        t = 0
        try:
            t = self._recode(coutfile, &vorbisfile, quality)
        finally:
            stdio.fclose(coutfile)
        return t


cdef int _writeToFile(void* coutfile, ogg.ogg_page* ogg_page) nogil:
    cdef size_t count
    count = ogg_page.header_len
    if stdio.fwrite(ogg_page.header, 1, count, <stdio.FILE*>coutfile) < count:
        return -1
    count = ogg_page.body_len
    if stdio.fwrite(ogg_page.body,   1, count, <stdio.FILE*>coutfile) < count:
        return -1
    return 0


@cython.final
cdef class FilelikeReader:
    cdef object read
    cdef object tell
    cdef object seek
    cdef object close
    cdef object exception

    def __cinit__(self, f):
        self.read  = f.read
        self.tell  = f.tell
        self.seek  = f.seek
        try:
            self.close = f.close
        except AttributeError:
            pass

    cdef void _storeException(self, exception):
        if self.exception is None:
            if exception is None:
                self.exception = sys.exc_info()
            else:
                self.exception = exception
    

cdef class VorbisFilelikeRecoder(_VorbisRecoder):
    cdef object f

    def __cinit__(self, f):
        if not hasattr(f, 'read') or \
               not hasattr(f, 'tell') or \
               not hasattr(f, 'seek'):
            raise TypeError, "Seekable file-like object required, got %s" % type(f)
        self.f = f
        self._write = _writeToFilelike

    def recode(self, f, int quality):
        cdef vorbis.OggVorbis_File vorbisfile
        cdef FilelikeReader reader
        write = f.write # make sure it's there
        if not callable(write):
            raise TypeError("writable file-like object required, got %s" % type(f).__name__)
        reader = FilelikeReader(self.f)

        result = vorbis.ov_open_callbacks(<void*>reader, &vorbisfile,
                                          NULL, 0, filelike_callbacks)
        if result == vorbis.OV_ENOTVORBIS:
            raise VorbisException("file is not a Vorbis file")
        elif result < 0:
            raise IOError("Error reading from file-like")

        t = self._recode(<void*>write, &vorbisfile, quality)

        if reader.exception is not None:
            raise reader.exception[0], reader.exception[1], reader.exception[2]

        return t


cdef int _writeToFilelike(void* outfile_write, ogg.ogg_page* ogg_page) with gil:
    try:
        write = <object>outfile_write
        write(ogg_page.header[:ogg_page.header_len])
        write(ogg_page.body[:ogg_page.body_len])
        return 0
    except:
        return -1


cdef size_t _readFilelike(void* dest, size_t size, size_t count,
                          void* creader) with gil:
    cdef FilelikeReader reader = <FilelikeReader>creader
    try:
        data = reader.read(size * count)
        if data is None:
            return 0
        elif isinstance(data, bytes):
            length = len(<bytes>data)
            if length > size * count:
                reader._storeException(
                    ValueError("Returned string is too long"))
            else:
                if length > 0:
                    memcpy(dest, <char*><bytes>data, length)
                return length
        else:
            reader._storeException(
                TypeError(
                    "Invalid return type from read(), bytes required, got %s"
                    % type(data.__name__)))
    except:
        reader._storeException(None)
    errno = -1  # propagate read error
    return 0


cdef int _seekFilelike(void* creader, ogg.ogg_int64_t offset, int whence) with gil:
    cdef FilelikeReader reader = <FilelikeReader>creader
    try:
        reader.seek(offset, whence)
        return 0
    except:
        reader._storeException(None)
    return -1


cdef long _tellFilelike(void* creader) with gil:
    cdef FilelikeReader reader = <FilelikeReader>creader
    try:
        return reader.tell()
    except:
        reader._storeException(None)
    return -1


cdef int _closeFilelike(void* creader) with gil:
    cdef FilelikeReader reader = <FilelikeReader>creader
    try:
        reader.seek(0)
        return 0
    except:
        reader._storeException(None)
    return -1


cdef void _splitStereo(char* decbuffer, float* channel1, float* channel2,
                       int samples) nogil:
    cdef int i
    for i in range(samples):
        channel1[i] = (
            ((decbuffer[i*4+1] << 8) | (0x00ff & <int>decbuffer[i*4]))
            ) / 32768.0

        channel2[i] = (
            ((decbuffer[i*4+3] << 8) | (0x00ff & <int>decbuffer[i*4+2]))
            ) / 32768.0


cdef int _initVorbisEncoder(ogg.ogg_stream_state* stream_state,
                            vorbis.vorbis_dsp_state* dsp_state,
                            vorbis.vorbis_info* vorbis_info,
                            int serialno):
    vorbis.vorbis_analysis_init(dsp_state, vorbis_info)
    return ogg.ogg_stream_init(stream_state, serialno)




cdef extern from "ogg/ogg.h" nogil:
    ctypedef long ogg_int64_t
    
    ctypedef struct oggpack_buffer:
        pass

    ctypedef struct ogg_page:
        unsigned char *header
        long header_len
        unsigned char *body
        long body_len

    ctypedef struct ogg_packet:
        pass

    ctypedef struct ogg_stream_state:
        pass

    cdef int ogg_stream_init(ogg_stream_state *os, int serialno)
    cdef int ogg_stream_clear(ogg_stream_state *os)
    cdef int ogg_stream_destroy(ogg_stream_state *os)

    cdef int ogg_stream_packetin(ogg_stream_state *os, ogg_packet *op)
    cdef int ogg_stream_pageout(ogg_stream_state *os, ogg_page *og)
    cdef int ogg_stream_flush(ogg_stream_state *os, ogg_page *og)

    cdef int ogg_page_eos(ogg_page *og)



from __future__ import absolute_import

import re
import io
import os
import sys
import os.path
from distutils.core import setup, Extension

with io.open("src/pyrogg.pyx", encoding="iso8859-1") as f:
    version = re.search('__version__\s*=\s*"([0-9.]+)"', f.read(1024)).group(1)

EXT_MODULE = "pyrogg"
setup_args = {}
DEFINES = []

LIBS = ['vorbis', 'vorbisfile', 'vorbisenc', 'ogg']
cflags = ['-Wall']
libs = ['-lm']
static_libs = []
static_libs_dir = os.environ.get("PYROGG_STATIC_LIBS")

if static_libs_dir:
    static_libs = [
        os.path.join(static_libs_dir, 'lib%s.a' % lib)
        for lib in LIBS
    ]
else:
    libs = ['-l' + lib for lib in LIBS] + libs

ext_file = os.path.join('src', EXT_MODULE)
pyx_source = ext_file + '.pyx'
c_source = ext_file + '.c'

try: sys.argv.remove('--without-assert')
except ValueError: pass
else: DEFINES.append( ('CYTHON_WITHOUT_ASSERTIONS', None) )

try: sys.argv.remove('--debug-gcc')
except ValueError: pass
else: cflags.append('-ggdb')

try: sys.argv.remove('--no-parallel')
except ValueError:
    if sys.platform == 'win32':
        cflags.append('/fopenmp')
        libs.append('/fopenmp')
    else:
        cflags.append('-fopenmp')
        libs.append('-fopenmp')

if os.environ.get('CFLAGS'):
    cflags += os.environ.get('CFLAGS').split()
if os.environ.get('LDFLAGS'):
    libs += os.environ.get('LDFLAGS').split()

extension = Extension(
    EXT_MODULE, [pyx_source],
    extra_compile_args=cflags,
    define_macros=DEFINES,
    extra_objects=static_libs or None,
    extra_link_args=libs,
)

if not os.path.exists(c_source) or '--recompile' in sys.argv:
    try: sys.argv.remove('--recompile')
    except ValueError: pass
    from Cython.Build import cythonize
    ext_modules = cythonize([extension])
else:
    extension.sources[0] = c_source
    ext_modules = [extension]


with open('README.rst') as f:
    long_description = f.read()

setup(
    name="pyrogg",
    version=version,
    author="pyrogg dev team",
    author_email="stefan_ml@behnel.de",
    maintainer="pyrogg dev team",
    maintainer_email="stefan_ml@behnel.de",
    url="https://github.com/scoder/pyrogg",

    description="Recode Ogg-Vorbis files to a different quality level",
    long_description=long_description,

    classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Intended Audience :: Information Technology',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Cython',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Operating System :: OS Independent',
    'Topic :: Multimedia :: Sound/Audio',
    ],

    package_dir = {'': 'src'},
#    packages = [''],
    ext_modules = ext_modules,
    **setup_args
)



import os
import glob
import sys
sys.path.insert(0, "src")

import pyrogg

QUALITY_LEVELS = tuple(range(0, 11))
TEST_FILE = "test.ogg"
TEST_DIR = "testoutput"


def output_filename(test_dir, quality, recoder_type):
    return os.path.join(test_dir, 'out_%02d_%s.ogg' % (
        quality+1, 'cfile' if recoder_type == 'file' else 'pfile'))


def main(test_dir=TEST_DIR):
    files1, files2 = [], []
    recoder = pyrogg.VorbisFileRecoder(TEST_FILE)
    for quality in QUALITY_LEVELS:
        print("Encoding in quality %+d" % quality)
        filename = output_filename(test_dir, quality, 'file')
        files1.append(filename)
        print("time: %.2f" % recoder.recode(filename, quality))

    for quality in QUALITY_LEVELS:
        with open(TEST_FILE, 'rb') as f:
            recoder = pyrogg.VorbisFilelikeRecoder(f)
            print("Encoding in quality %+d" % quality)
            filename = output_filename(test_dir, quality, 'filelike')
            files2.append(filename)
            with open(filename, 'wb') as fout:
                print("time: %.2f" % recoder.recode(fout, quality))

    for quality, f1, f2 in zip(QUALITY_LEVELS, files1, files2):
        with open(f1, 'rb') as fresult1:
            with open(f2, 'rb') as fresult2:
                assert fresult1.read() == fresult2.read(), \
                    "mismatch at quality level %d" % quality


if __name__ == '__main__':
    if not os.path.isfile(TEST_FILE):
        print("Please provide a test file at '%s'" % TEST_FILE)
        sys.exit(1)
    test_dir = os.path.abspath(TEST_DIR)
    if not os.path.isdir(test_dir):
        os.makedirs(test_dir)
    try:
        main(test_dir)
    finally:
        for filepath in glob.iglob(os.path.join(test_dir, 'out_*.ogg')):
            os.remove(filepath)
